#!/usr/bin/python

import os
import sys
import argparse
import createrepo
from retrace import *

if __name__ == "__main__":
    argparser = argparse.ArgumentParser(description="Retrace Server repository downloader")
    argparser.add_argument("distribution", type=str, help="Distribution name")
    argparser.add_argument("version", type=str, help="Release version")
    argparser.add_argument("architecture", type=str, help="CPU architecture")
    args = argparser.parse_args()

    distribution = args.distribution
    version = args.version
    arch = args.architecture

    if arch == "i686":
        arch = "i386"

    plugin = None
    for iplugin in PLUGINS:
        if iplugin.distribution == distribution:
            plugin = iplugin
            break

    if not plugin:
        print "Unknown distribution: '%s'" % distribution
        sys.exit(1)

    lock = "/var/lock/subsys/abrt-retrace-%s-%s-%s" % (distribution, version, arch)

    if os.path.isfile(lock):
        print "Another process with repository download is running."
        sys.exit(2)

    try:
        open(lock, "w").close()
    except Exception as ex:
        print "Unable to set lock: %s." % ex
        sys.exit(3)

    null = open("/dev/null", "w")

    for repo in plugin.repos:
        retcode = -1
        for mirror in repo:
            repourl = mirror.replace("$ARCH", arch).replace("$VER", version)
            targetdir = "%s/%s-%s-%s" % (CONFIG["RepoDir"], distribution, version, arch)

            print "Running rsync on '%s'..." % repourl,
            sys.stdout.flush()

            if repourl.startswith("rsync://"):
                files = [repourl]
            else:
                files = []
                try:
                    for package in os.listdir(repourl):
                        files.append("%s/%s" % (repourl, package))
                except Exception as ex:
                    print "Error: %s. Trying another mirror..." % ex
                    continue

            pipe = Popen(["rsync"] + files + [targetdir], stdout=null, stderr=null)
            pipe.wait()
            retcode = pipe.returncode

            if retcode == 0:
                print "OK"
                break

            print "Error. Trying another mirror..."

        if retcode != 0:
            print "No more mirrors to try."

    null.close()

    config = createrepo.MetaDataConfig()
    config.directory = targetdir
    config.quiet = True
    generator = createrepo.MetaDataGenerator(config)
    generator.doPkgMetadata()

    try:
        os.unlink(lock)
    except Exception as ex:
        print "Unable to remove lock: %s." % ex
        sys.exit(4)
